# Generated by Django 2.0.5 on 2018-05-22 20:48

from django.db import migrations, models

def move_catalog_data(apps, schema_editor):
    # Migrate the version information from the JSON data to the new
    # dedicated fields.
    Module = apps.get_model("guidedmodules", "Module")
    for app_module in Module.objects\
        .filter(module_name="app")\
        .exclude(app=None)\
        .select_related("app"):

        for field1, field2 in (('version_number', 'version'), ('version_name', 'version-name')):
            if field2 in app_module.app.catalog_metadata:
                setattr(app_module.app, field1, app_module.app.catalog_metadata[field2])
                del app_module.app.catalog_metadata[field2]

        app_module.app.save(update_fields=['version_number', 'version_name'])

def restore_catalog_data(apps, schema_editor):
    # Migrate the version information back to the JSON data.
    Module = apps.get_model("guidedmodules", "Module")
    for app_module in Module.objects\
        .filter(module_name="app")\
        .exclude(app=None)\
        .select_related("app"):

        for field1, field2 in (('version_number', 'version'), ('version_name', 'version-name')):
            if getattr(app_module.app, field1, None):
                app_module.app.catalog_metadata[field2] = getattr(app_module.app, field1)

        app_module.app.save(update_fields=['catalog_metadata'])

class Migration(migrations.Migration):

    dependencies = [
        ('guidedmodules', '0044_appinstance_catalog_metadata'),
    ]

    operations = [
        migrations.AddField(
            model_name='appinstance',
            name='version_name',
            field=models.CharField(blank=True, help_text='The name of this version/release of the compliance app.', max_length=128, null=True),
        ),
        migrations.AddField(
            model_name='appinstance',
            name='version_number',
            field=models.CharField(blank=True, help_text='The version number of the compliance app.', max_length=128, null=True),
        ),
        migrations.RunPython(move_catalog_data, restore_catalog_data),
    ]
